@@ -72,6 +72,7 @@ WASH_N_FILL_DOCK = [
     RoborockDockTypeCode.p10_pro_dock,
     RoborockDockTypeCode.s8_maxv_ultra_dock,
     RoborockDockTypeCode.qrevo_s_dock,
+    RoborockDockTypeCode.saros_r10_dock,
     RoborockDockTypeCode.qrevo_curv_dock,
 ]
 RT = TypeVar("RT", bound=RoborockBase)
@@ -335,6 +336,10 @@ class RoborockClientV1(RoborockClient, ABC):
             return [ServerTimer(*server_timers)]
         return []
 
+    async def load_multi_map(self, map_flag: int) -> None:
+        """Load the map into the vacuum's memory."""
+        await self.send_command(RoborockCommand.LOAD_MULTI_MAP, [map_flag])
+
     def _get_payload(
         self,
         method: RoborockCommand | str,
