@@ -22,6 +22,7 @@ from roborock.exceptions import (
     RoborockMissingParameters,
     RoborockNoUserAgreement,
     RoborockTooFrequentCodeRequests,
+    RoborockTooManyRequest,
     RoborockUrlException,
 )
 
@@ -29,16 +30,19 @@ _LOGGER = logging.getLogger(__name__)
 
 
 class RoborockApiClient:
-    def __init__(self, username: str, base_url=None) -> None:
+    def __init__(self, username: str, base_url=None, session: aiohttp.ClientSession | None = None) -> None:
         """Sample API Client."""
         self._username = username
         self._default_url = "https://euiot.roborock.com"
         self.base_url = base_url
         self._device_identifier = secrets.token_urlsafe(16)
+        if session is None:
+            session = aiohttp.ClientSession()
+        self.session = session
 
     async def _get_base_url(self) -> str:
         if not self.base_url:
-            url_request = PreparedRequest(self._default_url)
+            url_request = PreparedRequest(self._default_url, self.session)
             response = await url_request.request(
                 "post",
                 "/api/v1/getUrlByEmail",
@@ -54,6 +58,8 @@ class RoborockApiClient:
                     raise RoborockMissingParameters(
                         "You are missing parameters for this request, are you sure you " "entered your username?"
                     )
+                elif response_code == 9002:
+                    raise RoborockTooManyRequest("Please temporarily disable making requests and try again later.")
                 raise RoborockUrlException(f"error code: {response_code} msg: {response.get('error')}")
             response_data = response.get("data")
             if response_data is None:
@@ -110,7 +116,7 @@ class RoborockApiClient:
         ):
             raise RoborockException("Your userdata is missing critical attributes.")
         base_url = user_data.rriot.r.a
-        prepare_request = PreparedRequest(base_url)
+        prepare_request = PreparedRequest(base_url, self.session)
         hid = await self._get_home_id(user_data)
 
         data = FormData()
@@ -148,7 +154,7 @@ class RoborockApiClient:
         ):
             raise RoborockException("Your userdata is missing critical attributes.")
         base_url = user_data.rriot.r.a
-        add_device_request = PreparedRequest(base_url)
+        add_device_request = PreparedRequest(base_url, self.session)
 
         add_device_response = await add_device_request.request(
             "GET",
@@ -173,7 +179,7 @@ class RoborockApiClient:
     async def request_code(self) -> None:
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
-        code_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        code_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
 
         code_response = await code_request.request(
             "post",
@@ -198,7 +204,7 @@ class RoborockApiClient:
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
 
-        login_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        login_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
         login_response = await login_request.request(
             "post",
             "/api/v1/login",
@@ -236,7 +242,7 @@ class RoborockApiClient:
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
 
-        login_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        login_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
         login_response = await login_request.request(
             "post",
             "/api/v1/loginWithCode",
@@ -267,7 +273,7 @@ class RoborockApiClient:
     async def _get_home_id(self, user_data: UserData):
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
-        home_id_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        home_id_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
         home_id_response = await home_id_request.request(
             "get",
             "/api/v1/getHomeDetail",
@@ -282,7 +288,7 @@ class RoborockApiClient:
                 )
             raise RoborockException(f"{home_id_response.get('msg')} - response code: {home_id_response.get('code')}")
 
-        return home_id_response["data"].get("rrHomeId")
+        return home_id_response["data"]["rrHomeId"]
 
     async def get_home_data(self, user_data: UserData) -> HomeData:
         rriot = user_data.rriot
@@ -293,6 +299,7 @@ class RoborockApiClient:
             raise RoborockException("Missing field 'a' in rriot reference")
         home_request = PreparedRequest(
             rriot.r.a,
+            self.session,
             {
                 "Authorization": self._get_hawk_authentication(rriot, f"/user/homes/{str(home_id)}"),
             },
@@ -316,6 +323,7 @@ class RoborockApiClient:
             raise RoborockException("Missing field 'a' in rriot reference")
         home_request = PreparedRequest(
             rriot.r.a,
+            self.session,
             {
                 "Authorization": self._get_hawk_authentication(rriot, "/v2/user/homes/" + str(home_id)),
             },
@@ -329,6 +337,26 @@ class RoborockApiClient:
         else:
             raise RoborockException("home_response result was an unexpected type")
 
+    async def get_home_data_v3(self, user_data: UserData) -> HomeData:
+        """This is the same as get_home_data, but uses a different endpoint and includes non-robotic vacuums."""
+        rriot = user_data.rriot
+        home_id = await self._get_home_id(user_data)
+        if rriot.r.a is None:
+            raise RoborockException("Missing field 'a' in rriot reference")
+        home_request = PreparedRequest(
+            rriot.r.a,
+            {
+                "Authorization": self._get_hawk_authentication(rriot, "/v3/user/homes/" + home_id),
+            },
+        )
+        home_response = await home_request.request("get", "/v3/user/homes/" + home_id)
+        if not home_response.get("success"):
+            raise RoborockException(home_response)
+        home_data = home_response.get("result")
+        if isinstance(home_data, dict):
+            return HomeData.from_dict(home_data)
+        raise RoborockException(f"home_response result was an unexpected type: {home_data}")
+
     async def get_rooms(self, user_data: UserData, home_id: int | None = None) -> list[HomeDataRoom]:
         rriot = user_data.rriot
         if rriot is None:
@@ -339,6 +367,7 @@ class RoborockApiClient:
             raise RoborockException("Missing field 'a' in rriot reference")
         room_request = PreparedRequest(
             rriot.r.a,
+            self.session,
             {
                 "Authorization": self._get_hawk_authentication(rriot, "/v2/user/homes/" + str(home_id)),
             },
@@ -363,6 +392,7 @@ class RoborockApiClient:
             raise RoborockException("Missing field 'a' in rriot reference")
         scenes_request = PreparedRequest(
             rriot.r.a,
+            self.session,
             {
                 "Authorization": self._get_hawk_authentication(rriot, f"/user/scene/device/{str(device_id)}"),
             },
@@ -384,6 +414,7 @@ class RoborockApiClient:
             raise RoborockException("Missing field 'a' in rriot reference")
         execute_scene_request = PreparedRequest(
             rriot.r.a,
+            self.session,
             {
                 "Authorization": self._get_hawk_authentication(rriot, f"/user/scene/{str(scene_id)}/execute"),
             },
@@ -396,7 +427,7 @@ class RoborockApiClient:
         """Gets all products and their schemas, good for determining status codes and model numbers."""
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
-        product_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        product_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
         product_response = await product_request.request(
             "get",
             "/api/v4/product",
@@ -414,7 +445,7 @@ class RoborockApiClient:
     async def download_code(self, user_data: UserData, product_id: int):
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
-        product_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        product_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
         request = {"apilevel": 99999, "productids": [product_id], "type": 2}
         response = await product_request.request(
             "post",
@@ -427,7 +458,7 @@ class RoborockApiClient:
     async def download_category_code(self, user_data: UserData):
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
-        product_request = PreparedRequest(base_url, {"header_clientid": header_clientid})
+        product_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
         response = await product_request.request(
             "get",
             "api/v1/plugins?apiLevel=99999&type=2",
@@ -439,25 +470,27 @@ class RoborockApiClient:
 
 
 class PreparedRequest:
-    def __init__(self, base_url: str, base_headers: dict | None = None) -> None:
+    def __init__(self, base_url: str, session: aiohttp.ClientSession, base_headers: dict | None = None) -> None:
         self.base_url = base_url
         self.base_headers = base_headers or {}
+        self.session = session
 
     async def request(self, method: str, url: str, params=None, data=None, headers=None, json=None) -> dict:
         _url = "/".join(s.strip("/") for s in [self.base_url, url])
         _headers = {**self.base_headers, **(headers or {})}
-        async with aiohttp.ClientSession() as session:
+        try:
+            async with self.session.request(
+                method, _url, params=params, data=data, headers=_headers, json=json
+            ) as resp:
+                return await resp.json()
+        except ContentTypeError as err:
+            """If we get an error, lets log everything for debugging."""
             try:
-                async with session.request(method, _url, params=params, data=data, headers=_headers, json=json) as resp:
-                    return await resp.json()
-            except ContentTypeError as err:
-                """If we get an error, lets log everything for debugging."""
-                try:
-                    resp_json = await resp.json(content_type=None)
-                    _LOGGER.info("Resp: %s", resp_json)
-                except ContentTypeError as err_2:
-                    _LOGGER.info(err_2)
-                resp_raw = await resp.read()
-                _LOGGER.info("Resp raw: %s", resp_raw)
-                # Still raise the err so that it's clear it failed.
-                raise err
+                resp_json = await resp.json(content_type=None)
+                _LOGGER.info("Resp: %s", resp_json)
+            except ContentTypeError as err_2:
+                _LOGGER.info(err_2)
+            resp_raw = await resp.read()
+            _LOGGER.info("Resp raw: %s", resp_raw)
+            # Still raise the err so that it's clear it failed.
+            raise err
